@@ -7,7 +7,6 @@ from aiosignalrcore.hub.base_hub_connection import BaseHubConnection  # type: ig
 from aiosignalrcore.hub_connection_builder import HubConnectionBuilder  # type: ignore
 from aiosignalrcore.messages.completion_message import CompletionMessage  # type: ignore
 from aiosignalrcore.transport.websockets.connection import ConnectionState  # type: ignore
-from pyee import AsyncIOEventEmitter  # type: ignore
 
 from dipdup.config import (
     BigMapIndexConfig,
@@ -16,6 +15,7 @@ from dipdup.config import (
     OperationHandlerOriginationPatternConfig,
     OperationIndexConfig,
 )
+from dipdup.datasources.datasource import IndexDatasource
 from dipdup.datasources.proxy import DatasourceRequestProxy
 from dipdup.datasources.tzkt.enums import TzktMessageType
 from dipdup.models import BigMapAction, BigMapData, OperationData
@@ -254,7 +254,7 @@ class BigMapFetcher:
             yield big_maps[0].level, big_maps[: i + 1]
 
 
-class TzktDatasource(AsyncIOEventEmitter):
+class TzktDatasource(IndexDatasource):
     """Bridge between REST/WS TzKT endpoints and DipDup.
 
     * Converts raw API data to models
@@ -452,7 +452,7 @@ class TzktDatasource(AsyncIOEventEmitter):
         else:
             raise NotImplementedError(f'Index kind `{index_config.kind}` is not supported')
 
-        await self.on_connect()
+        await self._on_connect()
 
     def _get_client(self) -> BaseHubConnection:
         """Create SignalR client, register message callbacks"""
@@ -471,10 +471,10 @@ class TzktDatasource(AsyncIOEventEmitter):
                 )
             ).build()
 
-            self._client.on_open(self.on_connect)
-            self._client.on_error(self.on_error)
-            self._client.on('operations', self.on_operation_message)
-            self._client.on('bigmaps', self.on_big_map_message)
+            self._client.on_open(self._on_connect)
+            self._client.on_error(self._on_error)
+            self._client.on('operations', self._on_operation_message)
+            self._client.on('bigmaps', self._on_big_map_message)
 
         return self._client
 
@@ -485,7 +485,7 @@ class TzktDatasource(AsyncIOEventEmitter):
         self._logger.info('Starting websocket client')
         await self._get_client().start()
 
-    async def on_connect(self) -> None:
+    async def _on_connect(self) -> None:
         """Subscribe to all required channels on established WS connection"""
         if self._get_client().transport.state != ConnectionState.connected:
             return
@@ -499,7 +499,7 @@ class TzktDatasource(AsyncIOEventEmitter):
         for address, paths in self._big_map_subscriptions.items():
             await self.subscribe_to_big_maps(address, paths)
 
-    def on_error(self, message: CompletionMessage) -> NoReturn:
+    def _on_error(self, message: CompletionMessage) -> NoReturn:
         """Raise exception from WS server's error message"""
         raise Exception(message.error)
 
@@ -554,7 +554,7 @@ class TzktDatasource(AsyncIOEventEmitter):
                 ],
             )
 
-    async def on_operation_message(
+    async def _on_operation_message(
         self,
         message: List[Dict[str, Any]],
     ) -> None:
@@ -577,18 +577,19 @@ class TzktDatasource(AsyncIOEventEmitter):
                     if operation.status != 'applied':
                         continue
                     operations.append(operation)
-                self.emit("operations", operations)
+                self.emit_operations(operations)
 
             elif message_type == TzktMessageType.REORG:
-                self.emit("rollback", self.level, current_level)
+                if self.level is None:
+                    raise RuntimeError
+                self.emit_rollback(self.level, current_level)
 
             else:
                 raise NotImplementedError
 
-    async def on_big_map_message(
+    async def _on_big_map_message(
         self,
         message: List[Dict[str, Any]],
-        sync=False,
     ) -> None:
         """Parse and emit raw big map diffs from WS"""
         for item in message:
@@ -606,10 +607,12 @@ class TzktDatasource(AsyncIOEventEmitter):
                 for big_map_json in item['data']:
                     big_map = self.convert_big_map(big_map_json)
                     big_maps.append(big_map)
-                self.emit("big_maps", big_maps)
+                self.emit_big_maps(big_maps)
 
             elif message_type == TzktMessageType.REORG:
-                self.emit("rollback", self.level, current_level)
+                if self.level is None:
+                    raise RuntimeError
+                self.emit_rollback(self.level, current_level)
 
             else:
                 raise NotImplementedError
