@@ -1,7 +1,6 @@
 import asyncio
 import hashlib
 import logging
-from functools import partial
 from os.path import join
 from posix import listdir
 from typing import Dict, List, cast
@@ -26,18 +25,19 @@ from dipdup.config import (
     StaticTemplateConfig,
     TzktDatasourceConfig,
 )
-from dipdup.context import RollbackHandlerContext
+from dipdup.context import DipDupContext, RollbackHandlerContext
 from dipdup.datasources import DatasourceT
 from dipdup.datasources.bcd.datasource import BcdDatasource
+from dipdup.datasources.datasource import IndexDatasource
 from dipdup.datasources.tzkt.datasource import TzktDatasource
 from dipdup.exceptions import ConfigurationError, HandlerImportError
 from dipdup.hasura import configure_hasura
-from dipdup.index import BigMapIndex, HandlerContext, Index, OperationIndex
+from dipdup.index import BigMapIndex, Index, OperationIndex
 from dipdup.models import BigMapData, IndexType, OperationData, State
 
 
 class IndexDispatcher:
-    def __init__(self, ctx: HandlerContext) -> None:
+    def __init__(self, ctx: DipDupContext) -> None:
         self._ctx = ctx
 
         self._logger = logging.getLogger(__name__)
@@ -82,21 +82,21 @@ class IndexDispatcher:
 
         self._ctx.reset()
 
-    async def dispatch_operations(self, operations: List[OperationData]) -> None:
+    async def dispatch_operations(self, datasource: TzktDatasource, operations: List[OperationData]) -> None:
         assert len(set(op.level for op in operations)) == 1
         level = operations[0].level
         for index in self._indexes.values():
-            if isinstance(index, OperationIndex):
+            if isinstance(index, OperationIndex) and index.datasource == datasource:
                 index.push(level, operations)
 
-    async def dispatch_big_maps(self, big_maps: List[BigMapData]) -> None:
+    async def dispatch_big_maps(self, datasource: TzktDatasource, big_maps: List[BigMapData]) -> None:
         assert len(set(op.level for op in big_maps)) == 1
         level = big_maps[0].level
         for index in self._indexes.values():
-            if isinstance(index, BigMapIndex):
+            if isinstance(index, BigMapIndex) and index.datasource == datasource:
                 index.push(level, big_maps)
 
-    async def _rollback(self, datasource: str, from_level: int, to_level: int) -> None:
+    async def _rollback(self, datasource: TzktDatasource, from_level: int, to_level: int) -> None:
         logger = utils.FormattedLogger(ROLLBACK_HANDLER)
         rollback_fn = self._ctx.config.get_rollback_fn()
         ctx = RollbackHandlerContext(
@@ -111,12 +111,12 @@ class IndexDispatcher:
 
     async def run(self, oneshot=False) -> None:
         self._logger.info('Starting index dispatcher')
-        for name, datasource in self._ctx.datasources.items():
-            if not isinstance(datasource, TzktDatasource):
+        for datasource in self._ctx.datasources.values():
+            if not isinstance(datasource, IndexDatasource):
                 continue
-            datasource.on('operations', self.dispatch_operations)
-            datasource.on('big_maps', self.dispatch_big_maps)
-            datasource.on('rollback', partial(self._rollback, datasource=name))
+            datasource.on_operations(self.dispatch_operations)
+            datasource.on_big_maps(self.dispatch_big_maps)
+            datasource.on_rollback(self._rollback)
 
         self._ctx.commit()
 
@@ -143,11 +143,9 @@ class DipDup:
         self._config = config
         self._datasources: Dict[str, DatasourceT] = {}
         self._datasources_by_config: Dict[DatasourceConfigT, DatasourceT] = {}
-        self._ctx = HandlerContext(
+        self._ctx = DipDupContext(
             config=self._config,
             datasources=self._datasources,
-            logger=utils.FormattedLogger(__name__),
-            template_values=None,
         )
         self._index_dispatcher = IndexDispatcher(self._ctx)
 
