@@ -16,7 +16,7 @@ from dipdup.config import (
     OperationIndexConfig,
     OperationType,
 )
-from dipdup.context import HandlerContext
+from dipdup.context import DipDupContext, HandlerContext
 from dipdup.datasources.tzkt.datasource import BigMapFetcher, OperationFetcher, TzktDatasource
 from dipdup.models import BigMapAction, BigMapData, BigMapDiff, OperationData, Origination, State, TemporaryState, Transaction
 from dipdup.utils import FormattedLogger, in_global_transaction
@@ -25,7 +25,7 @@ OperationGroup = namedtuple('OperationGroup', ('hash', 'counter'))
 
 
 class Index:
-    def __init__(self, ctx: HandlerContext, config: IndexConfigTemplateT, datasource: TzktDatasource) -> None:
+    def __init__(self, ctx: DipDupContext, config: IndexConfigTemplateT, datasource: TzktDatasource) -> None:
         self._ctx = ctx
         self._config = config
         self._datasource = datasource
@@ -33,6 +33,10 @@ class Index:
         self._logger = logging.getLogger(__name__)
         self._state: Optional[State] = None
 
+    @property
+    def datasource(self) -> TzktDatasource:
+        return self._datasource
+
     async def get_state(self) -> State:
         """Get state of index containing current level and config hash"""
         if self._state is None:
@@ -90,7 +94,7 @@ class Index:
 class OperationIndex(Index):
     _config: OperationIndexConfig
 
-    def __init__(self, ctx: HandlerContext, config: OperationIndexConfig, datasource: TzktDatasource) -> None:
+    def __init__(self, ctx: DipDupContext, config: OperationIndexConfig, datasource: TzktDatasource) -> None:
         super().__init__(ctx, config, datasource)
         self._queue: Deque[Tuple[int, List[OperationData]]] = deque()
         self._contract_hashes: Dict[str, Tuple[str, str]] = {}
@@ -283,6 +287,7 @@ class OperationIndex(Index):
             config=self._ctx.config,
             logger=logger,
             template_values=self._config.template_values,
+            datasource=self.datasource,
         )
 
         await handler_config.callback_fn(handler_context, *args)
@@ -325,7 +330,7 @@ class OperationIndex(Index):
 class BigMapIndex(Index):
     _config: BigMapIndexConfig
 
-    def __init__(self, ctx: HandlerContext, config: BigMapIndexConfig, datasource: TzktDatasource) -> None:
+    def __init__(self, ctx: DipDupContext, config: BigMapIndexConfig, datasource: TzktDatasource) -> None:
         super().__init__(ctx, config, datasource)
         self._queue: Deque[Tuple[int, List[BigMapData]]] = deque()
 
@@ -423,6 +428,7 @@ class BigMapIndex(Index):
             config=self._ctx.config,
             logger=logger,
             template_values=self._config.template_values,
+            datasource=self.datasource,
         )
 
         await handler_config.callback_fn(handler_context, big_map_context)
